#include <SoftModem.h>
#include <ctype.h>

#define DIR_PIN 2
#define STEP_PIN 9
#define BTN_PIN 12
 
#define LED_PIN_1 A2
#define LED_PIN_2 A3
#define LED_PIN_3 A4
#define LED_PIN_4 A5

#define FULL_ROTATION_AMT 500
#define NUM_ROTATIONS 15
#define GEAR_RATIO 10

// STEPPER MOTOR INPUTS
#define A 4
#define B 5
#define C 7
#define D 8


SoftModem modem;

boolean rotating = false;
boolean cancelled = false;
int currentRotation = 0;
char val;
float rotAmt = FULL_ROTATION_AMT/NUM_ROTATIONS; // the number of degrees the stepper should turn for each increment

long duty = 50;
int waitMicroSeconds = 350;
int pulseCount = 5;

int buttonState = HIGH;  
int lastButtonState = LOW;
long lastDebounceTime = 0;
long debounceDelay = 20;
boolean spinCreated = false;

void setup()
{
  Serial.begin(9600);

  pinMode(BTN_PIN, INPUT_PULLUP);

  pinMode(LED_PIN_1, OUTPUT);
  pinMode(LED_PIN_2, OUTPUT);
  pinMode(LED_PIN_3, OUTPUT);
  pinMode(LED_PIN_4, OUTPUT);

  pinMode(A, OUTPUT);
  pinMode(B, OUTPUT);
  pinMode(C, OUTPUT);
  pinMode(D, OUTPUT);

  modem.begin();
}

void loop(){

  if(rotating == false){
    // keep two of the LED lights on when the turntable is powered
    digitalWrite(LED_PIN_3, HIGH);
    digitalWrite(LED_PIN_1, HIGH);
  }

  if(modem.available ()) {
    // RECEIVED SIGNAL FROM IPHONE 
    int input = modem.read ();
    Serial.println(input);

    if(input == 253 || input == 93){
      // begin spin - flash LEDs
      Serial.print(" start ");
      modem.write(0xFC); // send confirmation signal to app
      flashLEDsOnce();
      currentRotation = 0;
      rotating = true;
      cancelled = false;
      delay(1000);
    }else if(input==251 || input == 91){
      // stop arduino
      Serial.println(" stop ");
      rotating = false;
      cancelled = true;
      spinCreated = false;
      currentRotation = 0;
      modem.write(0xFE);
    }else if(input == 234 || input == 74){
      // setup signal received (for one-time setup when user first downloads app)
      flashLEDsOnce();
      modem.write(0xEC);
      rotate(rotAmt, false);
      rotate(rotAmt, true);
    }else if(input == 255){
      // spin has been created -> (used for enabling remote shutter button)
      spinCreated = true;
    }
  }
  
  if(rotating && !cancelled && currentRotation < NUM_ROTATIONS){
//    Serial.println(currentRotation+1);
    singleRotation();
    currentRotation++;
  }else{
    rotating = false;
    cancelled = true;
  }
  
  if(Serial.available()){
    // SIGNAL RECEIVED FROM SERIAL MONITOR / PROCESSING
      val = Serial.read();
      // read serial character from the serial monitor and send to arduino  
      // blink LED to show that signal was sent to iPhone
      ledsOn();
      delay(2);
      ledsOff();

      // KEYBOARD INPUT FOR DEBUGGING THROUGH PROCESSING
      if(val=='L'){
        // nudge the motor CW 
        ledsOn();
        rotate(rotAmt, false);
        ledsOff();
      }else if(val=='R'){
        // nudge the motor CCW 
        ledsOn();
        rotate(rotAmt, true);
        ledsOff();    
      }else if(val=='S'){
        // do a full 360 spin, stopping 15 times
        takePhoto();
        for(int i = 0; i<NUM_ROTATIONS; i++){
          ledsOn();
          // move the motor
          rotate(rotAmt, false);
          ledsOff();
          if(i!=NUM_ROTATIONS-1){
            takePhoto();
          }
        }
        flashLEDs();
      }else if(val=='T'){
        // do a full 360 spin, no stopping
        ledsOn();
        rotate(FULL_ROTATION_AMT, false);
        ledsOff();
      }else if(val=='P'){
        // take a photo
        takePhoto();
      }
    }
    
 // EASY BUTTON STUFF
  
  int reading = digitalRead(BTN_PIN);
  if(reading != lastButtonState){
  
    lastDebounceTime = millis();
  }
  
  if ((millis() - lastDebounceTime) > debounceDelay) {
    // whatever the reading is at, it's been there for longer
    // than the debounce delay, so take it as the actual current state:

    // if the button state has changed:
    if (reading != buttonState) {
      buttonState = reading;
  
      // only toggle the LED if the new button state is HIGH
      if (buttonState == HIGH && spinCreated) {
          Serial.println("write start to app");
           modem.write(0xFC); // send confirmation signal to app
           flashLEDsOnce();
           currentRotation = 0;
           rotating = true;
           cancelled = false;
           delay(1000);
      }
    }
  }
  
  lastButtonState = reading;

}

// rotate - steps is number of steps, direction is true for forward, false for backwards
void rotate(int steps, boolean forward){
  const int list[] = {1,11,2,12,3,13,4,14};
  for (int i = 0; i < steps; i++){
    duty = 50;
    if (forward)
      {for (int j = 0; j <= 7; j++){doStep(list[j]);}}
    else
      {for (int j = 7; j >= 0; j--){doStep(list[j]);}}
  }
}

void singleRotation(){
  rotating = true;
  ledsOn();
  rotate(rotAmt, false);
  ledsOff();
  takePhoto();
}

void takePhoto(){
  // time it takes to capture photo and add imageview in app
  delay(1950);
}


// LED FUNCTIONS
void ledsOn(){
  digitalWrite(LED_PIN_1, HIGH);
  digitalWrite(LED_PIN_2, HIGH);
  digitalWrite(LED_PIN_3, HIGH);
  digitalWrite(LED_PIN_4, HIGH);
}

void ledsOff(){
  digitalWrite(LED_PIN_1, LOW);
  digitalWrite(LED_PIN_2, LOW);
  digitalWrite(LED_PIN_3, LOW);
  digitalWrite(LED_PIN_4, LOW);
}  

void flashLEDs(){
  for(int i=0; i<3; i++){
    digitalWrite(LED_PIN_1, HIGH);
    digitalWrite(LED_PIN_2, HIGH);
    digitalWrite(LED_PIN_3, HIGH);
    digitalWrite(LED_PIN_4, HIGH);
    delay(300);
    digitalWrite(LED_PIN_1, LOW);
    digitalWrite(LED_PIN_2, LOW);
    digitalWrite(LED_PIN_3, LOW);
    digitalWrite(LED_PIN_4, LOW);
    delay(300);
  }  
}

void flashLEDsOnce(){
  digitalWrite(LED_PIN_1, HIGH);
  digitalWrite(LED_PIN_2, HIGH);
  digitalWrite(LED_PIN_3, HIGH);
  digitalWrite(LED_PIN_4, HIGH);
  delay(100);
  digitalWrite(LED_PIN_1, LOW);
  digitalWrite(LED_PIN_2, LOW);
  digitalWrite(LED_PIN_3, LOW);
  digitalWrite(LED_PIN_4, LOW);
}


// STEPPER MOTOR MICROSTEPPING FUNCTIONS
// main routine to microstep
void doStep(int st){
  
  long dt1 = waitMicroSeconds * duty / 100;
  long dt2 = waitMicroSeconds * (100-duty) / 100;

  for (int j = 0; j < pulseCount; j++){
    switch (st){
    case 1: one();break;
    case 2: two();break;
    case 3: three();break;
    case 4: four();break;
    case 11: oneB();break;
    case 12: twoB();break;
    case 13: threeB();break;
    case 14: fourB();break;

    case 21: one();break;
    case 22: two();break;
    case 23: three();break;
    case 24: four();break;
    case 31: oneB();break;
    case 32: twoB();break;
    case 33: threeB();break;
    case 34: fourB();break;

    }

    delayMicroseconds(dt1);

    switch (st){
    case 1: one();break;
    case 2: two();break;
    case 3: three();break;
    case 4: four();break;
    case 11: oneB();break;
    case 12: twoB();break;
    case 13: threeB();break;
    case 14: fourB();break;

    case 21: oneB();break;
    case 22: twoB();break;
    case 23: threeB();break;
    case 24: fourB();break;
    case 31: two();break;
    case 32: three();break;
    case 33: four();break;
    case 34: one();break;
    }
    delayMicroseconds(dt2);
    
  }
}


void one(){
  digitalWrite(A, HIGH);   
  digitalWrite(B, LOW);   
  digitalWrite(C, HIGH);   
  digitalWrite(D, LOW);   
}

void two(){
  digitalWrite(A, HIGH);   
  digitalWrite(B, LOW);   
  digitalWrite(C, LOW);   
  digitalWrite(D, HIGH);   
}

void three(){
  digitalWrite(A, LOW);   
  digitalWrite(B, HIGH);   
  digitalWrite(C, LOW);   
  digitalWrite(D, HIGH);   
}

void four(){
  digitalWrite(A, LOW);   
  digitalWrite(B, HIGH);   
  digitalWrite(C, HIGH);   
  digitalWrite(D, LOW);   
}

void oneB(){
  digitalWrite(A, HIGH);   
  digitalWrite(B, LOW);   
  digitalWrite(C, LOW);   
  digitalWrite(D, LOW);   
}

void twoB(){
  digitalWrite(A, LOW);   
  digitalWrite(B, LOW);   
  digitalWrite(C, LOW);   
  digitalWrite(D, HIGH);   
}

void threeB(){
  digitalWrite(A, LOW);   
  digitalWrite(B, HIGH);   
  digitalWrite(C, LOW);   
  digitalWrite(D, LOW);   
}

void fourB(){
  digitalWrite(A, LOW);   
  digitalWrite(B, LOW);   
  digitalWrite(C, HIGH);   
  digitalWrite(D, LOW);   
}


